@@ -460,7 +460,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
     }
 
     // do the connections of the option parameters
-    for (i = 1; i < d->m_optList.size(); ++i) {
+    for (i = 0; i < d->m_optList.size(); ++i) {
         //qDebug() << d->m_optList.at(i)->name();
         connect(d->m_optList.at(i), SIGNAL(optsNeedReload()), d, SLOT(optReload()));
         connect(d->m_optList.at(i), SIGNAL(valsNeedReload()), d, SLOT(scheduleValReload()));
@@ -691,7 +691,7 @@ void KSaneWidget::getOptVals(QMap <QString, QString> &opts)
     opts.clear();
     QString tmp;
 
-    for (int i = 1; i < d->m_optList.size(); i++) {
+    for (int i = 0; i < d->m_optList.size(); i++) {
         option = d->m_optList.at(i);
         if (option->getValue(tmp)) {
             opts[option->name()] = tmp;
@@ -723,14 +723,42 @@ int KSaneWidget::setOptVals(const QMap <QString, QString> &opts)
         return -1;
     }
 
+    QMap <QString, QString> optionMapCopy = opts;
+
     QString tmp;
     int i;
     int ret = 0;
 
+    const QString sourceOptionSaneName = QStringLiteral(SANE_NAME_SCAN_SOURCE);
+    const QString modeOptionSaneName   = QStringLiteral(SANE_NAME_SCAN_MODE);
+
+    KSaneOption *opt;
+
+    // Priorize source option
+    if (optionMapCopy.contains(sourceOptionSaneName)) {
+        if ((opt = d->getOption(sourceOptionSaneName)) != nullptr) {
+            if (opt->setValue(optionMapCopy[sourceOptionSaneName])) {
+                ret++;
+            }
+        }
+        optionMapCopy.remove(sourceOptionSaneName);
+    }
+
+    // Priorize mode option
+    if (optionMapCopy.contains(modeOptionSaneName)) {
+        if ((opt = d->getOption(modeOptionSaneName)) != nullptr) {
+            if (opt->setValue(optionMapCopy[modeOptionSaneName])) {
+                ret++;
+            }
+        }
+        optionMapCopy.remove(modeOptionSaneName);
+    }
+
+    // Update remaining options
     for (i = 0; i < d->m_optList.size(); i++) {
-        if (opts.contains(d->m_optList.at(i)->name())) {
-            tmp = opts[d->m_optList.at(i)->name()];
-            if (d->m_optList.at(i)->setValue(tmp) == false) {
+        if (optionMapCopy.contains(d->m_optList.at(i)->name())) {
+            tmp = optionMapCopy[d->m_optList.at(i)->name()];
+            if (d->m_optList.at(i)->setValue(tmp)) {
                 ret++;
             }
         }
@@ -757,8 +785,8 @@ int KSaneWidget::setOptVals(const QMap <QString, QString> &opts)
     }
 
     // special handling for non-sane option
-    if (opts.contains(InvetColorsOption)) {
-        tmp = opts[InvetColorsOption];
+    if (optionMapCopy.contains(InvetColorsOption)) {
+        tmp = optionMapCopy[InvetColorsOption];
         if ((tmp.compare(QStringLiteral("true"), Qt::CaseInsensitive) == 0) ||
                 (tmp.compare(QStringLiteral("1")) == 0)) {
             d->m_invertColors->setChecked(true);
@@ -852,7 +880,7 @@ float KSaneWidget::currentDPI()
             return value;
         }
     }
-    return 0.0;
+    return 0.0; // Failure to read DPI
 }
 
 float KSaneWidget::scanAreaWidth()
@@ -861,7 +889,12 @@ float KSaneWidget::scanAreaWidth()
     if (d->m_optBrX) {
         if (d->m_optBrX->getUnit() == SANE_UNIT_PIXEL) {
             d->m_optBrX->getMaxValue(result);
-            result = result / currentDPI() / 25.4;
+            float dpi = currentDPI();
+            if (dpi < 1) {
+                qDebug() << "Broken DPI value";
+                dpi = 1.0;
+            }
+            result = result / dpi / 25.4;
         } else if (d->m_optBrX->getUnit() == SANE_UNIT_MM) {
             d->m_optBrX->getMaxValue(result);
         }
@@ -875,7 +908,12 @@ float KSaneWidget::scanAreaHeight()
     if (d->m_optBrY) {
         if (d->m_optBrY->getUnit() == SANE_UNIT_PIXEL) {
             d->m_optBrY->getMaxValue(result);
-            result = result / currentDPI() / 25.4;
+            float dpi = currentDPI();
+            if (dpi < 1) {
+                qDebug() << "Broken DPI value";
+                dpi = 1.0;
+            }
+            result = result / dpi / 25.4;
         } else if (d->m_optBrY->getUnit() == SANE_UNIT_MM) {
             d->m_optBrY->getMaxValue(result);
         }
@@ -888,18 +926,15 @@ void KSaneWidget::setSelection(QPointF topLeft, QPointF bottomRight)
     if (!d->m_optBrX || !d->m_optBrY || !d->m_optTlX || !d->m_optTlY) {
         return;
     }
-    float xmax, ymax;
-    d->m_optBrX->getMaxValue(xmax);
-    d->m_optBrY->getMaxValue(ymax);
     if (topLeft.x() < 0.0 || topLeft.y() < 0.0 || bottomRight.x() < 0.0 || bottomRight.y() < 0.0) {
         d->m_previewViewer->clearActiveSelection();
         return;
     }
 
-    float tlxRatio = topLeft.x()/xmax;
-    float tlyRatio = topLeft.y()/ymax;
-    float brxRatio = bottomRight.x()/xmax;
-    float bryRatio = bottomRight.y()/ymax;
+    float tlxRatio = d->scanAreaToRatioX(topLeft.x());
+    float tlyRatio = d->scanAreaToRatioY(topLeft.y());
+    float brxRatio = d->scanAreaToRatioX(bottomRight.x());
+    float bryRatio = d->scanAreaToRatioX(bottomRight.y());
 
     d->m_previewViewer->setSelection(tlxRatio, tlyRatio, brxRatio, bryRatio);
 }
